@@ -8,6 +8,8 @@
 //! [Section docs](https://docs.rs/glyph_brush/latest/glyph_brush/struct.Section.html)
 //! for better understanding of adding and managing text.
 //!
+//! If you want to learn about GPU texture caching see [caching behaviour](#caching-behaviour).
+//!
 //! * Look trough [examples](https://github.com/Blatko1/wgpu_text/tree/master/examples).
 
 mod cache;
@@ -32,9 +34,8 @@ use glyph_brush::{
 };
 use pipeline::{Pipeline, Vertex};
 
-/// Marks scissor region and tries to fit it in inside the specified surface dimensions
-/// to avoid `wgpu` related rendering errors. `s_width` and `s_height` are most likely
-/// to be your windows width and height.
+/// Marks scissor region and can test how to fit itself inside the specified window dimensions
+/// to avoid `wgpu` related rendering errors.
 pub struct ScissorRegion {
     /// x coordinate of top left region point.
     pub x: u32,
@@ -47,32 +48,26 @@ pub struct ScissorRegion {
 
     /// Height of scissor region.
     pub height: u32,
-
-    /// Surface height.
-    pub s_width: u32,
-
-    /// Surface width.
-    pub s_height: u32,
 }
 
 impl ScissorRegion {
     /// Checks if the region is contained in surface bounds at all.
-    pub(crate) fn is_contained(&self) -> bool {
-        if self.x < self.s_width && self.y < self.s_height {
+    pub(crate) fn is_contained_in(&self, width: u32, height: u32) -> bool {
+        if self.x < width && self.y < height {
             return true;
         }
         false
     }
 
-    /// Gives available bounds paying attention to `s_width` and `s_height`.
-    pub(crate) fn available_bounds(&self) -> (u32, u32) {
-        let width = if (self.x + self.width) > self.s_width {
-            self.s_width - self.x
+    /// Gives available bounds paying attention to `width` and `height`.
+    pub(crate) fn available_bounds(&self, width: u32, height: u32) -> (u32, u32) {
+        let width = if (self.x + self.width) > width {
+            width - self.x
         } else {
             self.width
         };
-        let height = if (self.y + self.height) > self.s_height {
-            self.s_height - self.y
+        let height = if (self.y + self.height) > height {
+            height - self.y
         } else {
             self.height
         };
@@ -86,9 +81,14 @@ impl ScissorRegion {
 pub struct TextBrush<F = FontArc, H = DefaultSectionHasher> {
     inner: glyph_brush::GlyphBrush<Vertex, Extra, F, H>,
     pipeline: Pipeline,
+    depth_test: bool,
 }
 
-impl<F: Font + Sync, H: std::hash::BuildHasher> TextBrush<F, H> {
+impl<F, H> TextBrush<F, H>
+where
+    F: Font + Sync,
+    H: std::hash::BuildHasher,
+{
     /// Queues section for drawing. This should be called every frame for every section that is going to be drawn.
     ///
     /// This can be called multiple times for different sections that want to use the
@@ -106,6 +106,7 @@ impl<F: Font + Sync, H: std::hash::BuildHasher> TextBrush<F, H> {
         device: &wgpu::Device,
         view: &wgpu::TextureView,
         queue: &wgpu::Queue,
+        config: &wgpu::SurfaceConfiguration,
         region: Option<ScissorRegion>,
     ) -> wgpu::CommandBuffer {
         let mut brush_action;
@@ -148,36 +149,54 @@ impl<F: Font + Sync, H: std::hash::BuildHasher> TextBrush<F, H> {
             BrushAction::ReDraw => (),
         }
 
-        self.pipeline.draw(device, view, region)
+        let depth_view = Pipeline::depth_texture_view(device, config);
+        let depth = if self.depth_test {
+            Some(wgpu::RenderPassDepthStencilAttachment {
+                view: &depth_view,
+                depth_ops: Some(wgpu::Operations {
+                    load: wgpu::LoadOp::Clear(1.0),
+                    store: true,
+                }),
+                stencil_ops: None,
+            })
+        } else {
+            None
+        };
+
+        self.pipeline.draw(device, view, config, depth, region)
     }
 
     /// Draws all queued sections with [`queue`](#method.queue) function.
     ///
     /// Use [`TextBrush::draw_custom`] for more rendering options.
+    #[inline]
     pub fn draw(
         &mut self,
         device: &wgpu::Device,
         view: &wgpu::TextureView,
         queue: &wgpu::Queue,
+        config: &wgpu::SurfaceConfiguration,
     ) -> wgpu::CommandBuffer {
-        self.draw_queued(device, view, queue, None)
+        self.draw_queued(device, view, queue, config, None)
     }
 
     /// Draws all queued text with extra options.
     ///
     /// # Scissoring
     /// With scissoring, you can filter out each glyph fragment that crosses the given `region`.
+    #[inline]
     pub fn draw_custom<R>(
         &mut self,
         device: &wgpu::Device,
         view: &wgpu::TextureView,
         queue: &wgpu::Queue,
+        config: &wgpu::SurfaceConfiguration,
         region: Option<R>,
     ) -> wgpu::CommandBuffer
     where
         R: Into<ScissorRegion>,
     {
-        self.draw_queued(device, view, queue, region.map(|r| r.into()))
+        self.draw_queued(device, view, queue, config, region.map(|r| r.into()))
     }
 
     /// Resizes "_camera_". Updates default orthogonal view matrix
@@ -185,12 +204,35 @@ impl<F: Font + Sync, H: std::hash::BuildHasher> TextBrush<F, H> {
     ///
     /// Run this function whenever the surface is resized.
     /// _width_ and _height_ should be **surfaces** dimensions.
+    ///
+    /// **Matrix**:
+    /// ```rust
+    /// fn ortho(width: f32, height: f32) -> [f32; 16] {
+    ///     [
+    ///         2.0 / width, 0.0,          0.0, 0.0,
+    ///         0.0,        -2.0 / height, 0.0, 0.0,
+    ///         0.0,         0.0,          1.0, 0.0,
+    ///        -1.0,         1.0,          0.0, 1.0,
+    ///     ]
+    /// }
+    /// ```
     #[inline]
     pub fn resize_view(&mut self, width: f32, height: f32, queue: &wgpu::Queue) {
         let matrix = ortho(width, height);
         self.pipeline.update_matrix(matrix, queue);
     }
 
+    // TODO
+    /* /// Multiplies builtin orthogonal matrix with `matrix` and updates matrix buffer.
+    #[inline]
+    pub fn add_matrix<M>(&mut self, matrix: M, queue: &wgpu::Queue)
+    where
+        M: Into<[f32; 16]>,
+    {
+        let matrix = ;
+        self.pipeline.update_matrix(matrix.into(), queue);
+    }*/
+
     /// Provides your own matrix for rendering instead of default orthogonal view matrix.
     #[inline]
     pub fn custom_matrix<M>(&mut self, matrix: M, queue: &wgpu::Queue)
@@ -204,6 +246,7 @@ impl<F: Font + Sync, H: std::hash::BuildHasher> TextBrush<F, H> {
 /// Builder for [`TextBrush`].
 pub struct BrushBuilder<F, H = DefaultSectionHasher> {
     inner: glyph_brush::GlyphBrushBuilder<F, H>,
+    depth_test: bool,
 }
 
 impl BrushBuilder<()> {
@@ -231,13 +274,27 @@ impl BrushBuilder<()> {
     pub fn using_fonts<F: Font>(fonts: Vec<F>) -> BrushBuilder<F> {
         BrushBuilder {
             inner: glyph_brush::GlyphBrushBuilder::using_fonts(fonts),
+            depth_test: false,
         }
     }
 }
 
-impl<F: Font, H: std::hash::BuildHasher> BrushBuilder<F, H> {
+impl<F, H> BrushBuilder<F, H>
+where
+    F: Font,
+    H: std::hash::BuildHasher,
+{
     glyph_brush::delegate_glyph_brush_builder_fns!(inner);
 
+    /// Defaults to `false`. If set to true all text will be depth tested. 
+    /// Depth can be for each section can be set by z coordinate.
+    /// 
+    /// When enabled, section `z` coordinate should be in range 0.0 - 1.0 not including 1.0.
+    pub fn with_depth_testing(mut self, depth_test: bool) -> Self {
+        self.depth_test = depth_test;
+        self
+    }
+
     /// Builds a [`TextBrush`] consuming [`BrushBuilder`].
     pub fn build(
         self,
@@ -246,10 +303,28 @@ impl<F: Font, H: std::hash::BuildHasher> BrushBuilder<F, H> {
         width: f32,
         height: f32,
     ) -> TextBrush<F, H> {
+        let depth = if self.depth_test {
+            Some(Pipeline::depth_state())
+        } else {
+            None
+        };
+
         let inner = self.inner.build();
         let matrix = ortho(width, height);
-        let pipeline = Pipeline::new(device, render_format, inner.texture_dimensions(), matrix);
-        TextBrush { inner, pipeline }
+
+        let pipeline = Pipeline::new(
+            device,
+            render_format,
+            depth,
+            inner.texture_dimensions(),
+            matrix,
+        );
+
+        TextBrush {
+            inner,
+            pipeline,
+            depth_test: self.depth_test,
+        }
     }
 }
 
