@@ -18,9 +18,12 @@ pub struct Pipeline {
 }
 
 impl Pipeline {
+    const DEPTH_FORMAT: wgpu::TextureFormat = wgpu::TextureFormat::Depth32Float;
+
     pub fn new(
         device: &wgpu::Device,
         render_format: wgpu::TextureFormat,
+        depth_stencil: Option<wgpu::DepthStencilState>,
         tex_dimensions: (u32, u32),
         matrix: [f32; 16],
     ) -> Self {
@@ -54,7 +57,7 @@ impl Pipeline {
                 strip_index_format: Some(wgpu::IndexFormat::Uint16),
                 ..Default::default()
             },
-            depth_stencil: None,
+            depth_stencil,
             multisample: wgpu::MultisampleState::default(),
             fragment: Some(wgpu::FragmentState {
                 module: &shader,
@@ -115,8 +118,13 @@ impl Pipeline {
         &self,
         device: &wgpu::Device,
         view: &wgpu::TextureView,
+        config: &wgpu::SurfaceConfiguration,
+        depth_stencil_attachment: Option<wgpu::RenderPassDepthStencilAttachment>,
         region: Option<crate::ScissorRegion>,
     ) -> CommandBuffer {
+        let width = config.width;
+        let height = config.height;
+
         let mut encoder = device.create_command_encoder(&wgpu::CommandEncoderDescriptor {
             label: Some("wgpu-text Command Encoder"),
         });
@@ -132,16 +140,17 @@ impl Pipeline {
                         store: true,
                     },
                 }],
-                depth_stencil_attachment: None,
+                depth_stencil_attachment,
             });
 
             rpass.set_pipeline(&self.inner);
             rpass.set_vertex_buffer(0, self.vertex_buffer.slice(..));
             rpass.set_bind_group(0, &self.cache.bind_group, &[]);
 
+            // Region scissoring
             if let Some(r) = region {
-                if r.is_contained() {
-                    let (w, h) = r.available_bounds();
+                if r.is_contained_in(width, height) {
+                    let (w, h) = r.available_bounds(width, height);
                     rpass.set_scissor_rect(r.x, r.y, w, h);
                 }
             }
@@ -152,17 +161,51 @@ impl Pipeline {
         encoder.finish()
     }
 
+    #[inline]
     pub fn update_matrix(&mut self, matrix: [f32; 16], queue: &wgpu::Queue) {
         self.cache.update_matrix(matrix, queue);
     }
 
+    #[inline]
     pub fn update_texture(&mut self, size: Rectangle<u32>, data: &[u8], queue: &wgpu::Queue) {
         self.cache.update_texture(size, data, queue);
     }
 
+    #[inline]
     pub fn resize_texture(&mut self, device: &wgpu::Device, width: u32, height: u32) {
         self.cache.recreate_texture(device, width, height);
     }
+
+    pub fn depth_texture_view(
+        device: &wgpu::Device,
+        config: &wgpu::SurfaceConfiguration,
+    ) -> wgpu::TextureView {
+        let depth_texture = device.create_texture(&wgpu::TextureDescriptor {
+            size: wgpu::Extent3d {
+                width: config.width,
+                height: config.height,
+                depth_or_array_layers: 1,
+            },
+            mip_level_count: 1,
+            sample_count: 1,
+            dimension: wgpu::TextureDimension::D2,
+            format: Self::DEPTH_FORMAT,
+            usage: wgpu::TextureUsages::RENDER_ATTACHMENT,
+            label: Some("wgpu-text Depth Texture"),
+        });
+
+        depth_texture.create_view(&wgpu::TextureViewDescriptor::default())
+    }
+
+    pub fn depth_state() -> wgpu::DepthStencilState {
+        wgpu::DepthStencilState {
+            format: Pipeline::DEPTH_FORMAT,
+            depth_write_enabled: true,
+            depth_compare: wgpu::CompareFunction::Less,
+            stencil: wgpu::StencilState::default(),
+            bias: wgpu::DepthBiasState::default(),
+        }
+    }
 }
 
 #[repr(C)]
